#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include "cv.h"
#include "highgui.h"
#include <stdlib.h>
#include <stdio.h>
#endif

#include <math.h>
#include <ctype.h>

char infilename[255];
char outfilename[255];

IplImage *inimage = 0, *outimage = 0, *temp = 0, *gray = 0, *dx = 0, *dy = 0;

char isopt(char *arg)
{
	if (arg==NULL)
		return 0;

	if (strlen(arg) == 2)
		if (arg[0] == '-')
			if (isalpha(arg[1]))
				return arg[1];

	return 0;
}

int params(int argc, char *argv[])
{
	int i=1, co;
	char opt;

	while (i<argc)
	{
		if (opt=isopt(argv[i]))
		{
			switch(opt)
			{
				case 'i': 
					break;

				case 'v': 
					break;

				case 'o':
					// next argument
					i++;

					// is our argument?
					if ((i>=argc) || (isopt(argv[i])))
					{
						printf("Missing output filename!\n");
		
						return -1;
					}
					else
					{
						strcpy(outfilename, argv[i]);
						strcat(outfilename, ".bmp");
					}

					break;


				default: 
					printf("Unknown option!\n");
		
					return -1;

			}

		}
		else
		{
			printf("Option expected or unexpected paramater!\n");

			return -1;
		}

		i++;
	}
	

	return 1;	
}

// find all values > 0 and store them in list
int find(CvMat *image, CvSeq *oseq)
{
	int x, y;
	CvScalar scal, point;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] > 0 )
			{
				point.val[0] = x;
				point.val[1] = y;

				cvSeqPush( oseq, &point );
			}
		}

	return oseq->total;
}

// find specified value and store them in list
int findval(CvMat *image, CvSeq *oseq, int val)
{
	int x, y;
	CvScalar scal, point;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] == val )
			{
				point.val[0] = x;
				point.val[1] = y;

				cvSeqPush( oseq, &point );
			}
		}

	return oseq->total;
}

int maxval(CvMat *image)
{
	int x, y, lastmax = 0;
	CvScalar scal, point;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] > lastmax )
			{
				lastmax = scal.val[0];
			}
		}

	return lastmax;
}

void HoughCircles(CvArr *image_source)
{
	CvMemStorage *ostorage = cvCreateMemStorage(0), *mstorage = cvCreateMemStorage(0);
	CvSeq *oseq, *mseq;
	CvMat tmp, *hm, *image = (CvMat*)image_source;
	int i, j, k;
	CvScalar *point, scal;
	int R = 34, R2;
	int x, y, maxhmval = 0;
	int a;
	IplImage *elan = 0;

	image = cvGetMat(image, &tmp, 0, 0); 

	printf("Rows '%d' & cols '%d'\n", image->rows, image->cols);

	// list for 
	oseq = cvCreateSeq( 0, sizeof(CvSeq), sizeof(CvScalar), ostorage );
	
	// find all values > 0
	find(image, oseq);

	R2=R*R;

	// create hough matrix
	hm = cvCreateMat( image->rows, image->cols, CV_8UC1);

	printf("%d ", oseq->total );
	for ( i = 0; i < oseq->total; i++ )
	{


		point = (CvScalar*)cvGetSeqElem(oseq, i );
		x = point->val[1];
		y = point->val[0];

		for ( j = 0; j < image->rows; j++ ) { 

			a = round( x - sqrt( R2 - pow( y - j, 2 )));

			if ( a > 0)
			{
				scal = cvGet2D( hm, j, a );
				scal.val[0]=scal.val[0]+10;
				cvSet2D( hm, j, a, scal );
				cvSet2D( outimage, j, a, scal );
			}
		}
	}

	maxhmval=maxval( hm );

	mseq = cvCreateSeq( 0, sizeof(CvSeq), sizeof(CvScalar), mstorage );

	findval( hm, mseq, maxhmval );

	for ( i = 0; i < mseq->total; i++ )
	{
		point = (CvScalar*)cvGetSeqElem(mseq, i );
		printf("[%d, %d] ", point->val[0], point->val[1]);		
	}
}

int main(int argc, char *argv[])
{
	int i, h, s, hi, si, x, y, fx, fy;
	CvScalar scal;
	CvCapture *invid=0;
	CvSize framesize;
	double max;
	int med[64];
	int midpoint, size, mult;

	printf("CDECT\n\n");

	cvNamedWindow("CDECT", 1);

	/*if (params(argc, argv)==-1)
	{
		printf("Bad arguments, exiting\n");
		exit(1);
	}*/

	strcpy(infilename, "./../data/c34.png");

	printf("Input filename '%s'\n", infilename);
	printf("Output filename '%s'\n", outfilename);

	if( (inimage = cvLoadImage( infilename, CV_LOAD_IMAGE_COLOR )) == 0 )
	{
		printf("Image loading troubles\n");
		return -1;
	}


	framesize.width=inimage->width;
	framesize.height=inimage->height;

	printf("In image channels '%d'\n", inimage->nChannels);
	printf("Source size is '%d x %d'\n", framesize.width, framesize.height);


	gray = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);
	cvCvtColor(inimage, gray, CV_BGR2GRAY); 

	outimage = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);
	dx = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);
	dy = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);

	cvSobel( gray, dx, 1, 0, 3 );
	cvSobel( gray, dy, 0, 1, 3 );

//	cvSmooth( gray, gray, CV_GAUSSIAN, 9, 9 );
	cvZero( outimage );
	HoughCircles(gray);

	 
	//cvShowImage("CDECT", gray);
	cvShowImage("CDECT", outimage);
	//cvShowImage("CDECT", dx);
	//cvShowImage("CDECT", dy);
/*	if( cvSaveImage( outfilename, outimage) == 0 )
		return -1;
*/

	cvWaitKey(0);
	return 0;
}
