#ifdef _CH_
#pragma package <opencv>
#endif

#ifndef _EiC
#include "cv.h"
#include "highgui.h"
#include <stdlib.h>
#include <stdio.h>
#endif

#include <math.h>
#include <ctype.h>

#define WIN_PREPR "Preprocessing"
#define WIN_HOUGH "Hough circles"
#define WIN_KNOWN "Know circle radius"

#define TRACK_RADIUS "Circle radius"
#define TRACK_THRESHOLD "Hough threshold"

#define HSTEP 1

char infilename[255];
char outfilename[255];

IplImage *inimage = 0, *outimage = 0, *houghimage = 0, *gray = 0;

int radius=34, threshold=radius*5;
CvSeq *oseq;
CvMemStorage *ostorage;

int firstrun=1;

char isopt(char *arg)
{
	if (arg==NULL)
		return 0;

	if (strlen(arg) == 2)
		if (arg[0] == '-')
			if (isalpha(arg[1]))
				return arg[1];

	return 0;
}

int params(int argc, char *argv[])
{
	int i=1;
	char opt;

	while (i<argc)
	{
		if (opt=isopt(argv[i]))
		{
			switch(opt)
			{
				case 'i': 
					break;

				case 'v': 
					break;

				case 'o':
					// next argument
					i++;

					// is our argument?
					if ((i>=argc) || (isopt(argv[i])))
					{
						printf("Missing output filename!\n");
		
						return -1;
					}
					else
					{
						strcpy(outfilename, argv[i]);
						strcat(outfilename, ".bmp");
					}

					break;


				default: 
					printf("Unknown option!\n");
		
					return -1;

			}

		}
		else
		{
			printf("Option expected or unexpected paramater!\n");

			return -1;
		}

		i++;
	}
	

	return 1;	
}

// find all values > 0 and store them in list
int find(CvMat *image, CvSeq *oseq)
{
	int x, y;
	CvScalar scal;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] > 0 )
			{
				CvScalar point;

				point.val[0] = x;
				point.val[1] = y;

				cvSeqPush( oseq, &point );
			}
		}

	return oseq->total;
}

// find specified value and store them in list
int findval(CvMat *image, CvSeq *mseq, int val)
{
	int x, y;
	CvScalar scal;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] == val )
			{
				CvScalar point;

				point.val[0] = x;
				point.val[1] = y;

				cvSeqPush( mseq, &point );
			}
		}

	return mseq->total;
}

int maxval(CvMat *image)
{
	int x, y, lastmax = 0;
	CvScalar scal;

	for ( x = 0; x < image->rows; x++ )
		for ( y = 0; y < image->cols; y++ )
		{
			scal = cvGet2D( image, x, y );
			
			if ( scal.val[0] > lastmax )
			{
				lastmax = scal.val[0];
			}
		}

	return lastmax;
}

void HoughCirclesKR(CvArr *image_source)
{
	CvMemStorage *mstorage, *sstorage;
	CvSeq *mseq, *sseq;
	CvMat tmp, *hm, *image = (CvMat*)image_source;
	CvConnectedComp *cur_comp;
	int i, j;
	CvScalar *point, scal;
	CvPoint pointx;
	int R, R2;
	int x, y, maxhmval = 0;
	int a, max;

	image = cvGetMat(image, &tmp, 0, 0); 

	if (firstrun)
		find(image, oseq);

	firstrun=0;

	R=radius;
	
	mstorage = cvCreateMemStorage(0);
	sstorage = cvCreateMemStorage(1000);

	mseq = cvCreateSeq( 0, sizeof(CvSeq), sizeof(CvScalar), mstorage );
	
	// precalc R^2
	R2=R*R;

	// create hough matrix
	hm = cvCreateMat( image->rows, image->cols, CV_16UC1);

	cvZero(hm);

	// for every feature point
	for ( i = 0; i < oseq->total; i++ )
	{
		point = (CvScalar*)cvGetSeqElem(oseq, i );
		x = point->val[1];
		y = point->val[0];

		// for every row in image
		for ( j = 0; j < image->rows; j++ ) { 

			// find a parameter
			a = round( x - sqrt( R2 - pow( y - j, 2 )));

			// add to accumulator buffer
			if ( a > 0)
			{
				scal = cvGet2D( hm, j, a );
				scal.val[0]=scal.val[0]+HSTEP;
				cvSet2D( hm, j, a, scal );
				cvSet2D( houghimage, j, a, scal );
			}
		}
	}

	// find maxval
	maxhmval=maxval( hm );
	printf("maxhmval %d", maxhmval);

/*	for ( i = 0; i <= image->rows; i++ )
		for ( j = 0; j <= image->cols; j++ )
		{
			scal.val[0] = (hist[h][s]/maxhmval)*255;
			cvSet2D(outimage, i, j, scal);
		}
*/
	// find votes with maxval
	if (maxhmval>=threshold)
	{
		findval( hm, mseq, maxhmval );

		printf(" vals %d\n", mseq->total);

		// for every vote draw circle
		for ( i = 0; i < mseq->total; i++ )
		{
			point = (CvScalar*)cvGetSeqElem(mseq, i );

			pointx.x = point->val[1];
			pointx.y = point->val[0];


			cvLine(outimage, cvPoint(pointx.x, pointx.y-2), cvPoint(pointx.x, pointx.y+2), cvScalar(255)); 
			cvLine(outimage, cvPoint(pointx.x-2, pointx.y), cvPoint(pointx.x+2, pointx.y), cvScalar(255)); 


			cvCircle(outimage, pointx, radius, cvScalar(255), 2);
		}
	}
}

void track_radius(int pos)
{
	cvZero(houghimage);
	cvZero(outimage);

	outimage=cvCloneImage(gray);

	HoughCirclesKR(gray);
	 
	cvShowImage(WIN_HOUGH, houghimage);	


	cvShowImage(WIN_PREPR, gray);	
	cvShowImage(WIN_KNOWN, outimage);	
}


int main(int argc, char *argv[])
{	CvSize framesize;

	printf("CDECT\n\n");

	cvNamedWindow(WIN_PREPR, 1);
	cvCreateTrackbar(TRACK_RADIUS, WIN_PREPR, &radius, 100, &track_radius); 

	cvNamedWindow(WIN_KNOWN, 1);

	cvNamedWindow(WIN_HOUGH, 1);
	cvCreateTrackbar(TRACK_RADIUS, WIN_HOUGH, &threshold, 500, &track_radius); 


	ostorage = cvCreateMemStorage(0);
	oseq = cvCreateSeq( 0, sizeof(CvSeq), sizeof(CvScalar), ostorage );

	/*if (params(argc, argv)==-1)
	{
		printf("Bad arguments, exiting\n");
		exit(1);
	}*/

	strcpy(infilename, "./../data/c34x2.png");
	strcpy(outfilename, "./../data/out.png");

	printf("Input filename '%s'\n", infilename);
	printf("Output filename '%s'\n", outfilename);

	if( (inimage = cvLoadImage( infilename, CV_LOAD_IMAGE_COLOR )) == 0 )
	{
		printf("Image loading troubles\n");
		return -1;
	}


	framesize.width=inimage->width;
	framesize.height=inimage->height;

	printf("In image channels '%d'\n", inimage->nChannels);
	printf("Source size is '%d x %d'\n", framesize.width, framesize.height);


	gray = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);
	cvCvtColor(inimage, gray, CV_BGR2GRAY); 

	houghimage = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);
	outimage = cvCreateImage(cvSize(inimage->width, inimage->height), IPL_DEPTH_8U, 1);

	cvZero( outimage );

	HoughCirclesKR(gray);

	cvShowImage(WIN_PREPR, gray);
	cvShowImage(WIN_HOUGH, houghimage);
	cvShowImage(WIN_KNOWN, outimage);

	if( cvSaveImage( outfilename, outimage) == 0 )
		return -1;


	cvWaitKey(0);
	return 0;
}
